@@ -54,7 +54,7 @@ Examples
 >>> plt.show()
 """
 
-__version__ = "4.2.3"
+__version__ = "4.4.1"
 __author__ = "Paul Brodersen"
 __email__ = "paulbrodersen+netgraph@gmail.com"
 
@@ -69,12 +69,15 @@ from ._node_layout import (
     get_fruchterman_reingold_layout,
     get_sugiyama_layout,
     get_circular_layout,
+    get_linear_layout,
+    get_bipartite_layout,
     get_community_layout,
 )
 
 from ._edge_layout import (
     get_straight_edge_paths,
     get_curved_edge_paths,
+    get_arced_edge_paths,
     get_bundled_edge_paths,
 )
 
@@ -83,6 +86,13 @@ from ._interactive_variants import (
     EditableGraph,
 )
 
+from ._arcdiagram import (
+    BaseArcDiagram,
+    ArcDiagram,
+    InteractiveArcDiagram,
+    EditableArcDiagram,
+)
+
 from ._parser import parse_graph
 
 
@@ -91,14 +101,21 @@ __all__ = [
     'get_fruchterman_reingold_layout',
     'get_sugiyama_layout',
     'get_circular_layout',
+    'get_linear_layout',
+    'get_bipartite_layout',
     'get_community_layout',
     'get_straight_edge_paths',
     'get_curved_edge_paths',
     'get_bundled_edge_paths',
+    'get_arced_edge_paths',
     'Basegraph',
     'Graph',
     'InteractiveGraph',
     'MutableGraph',
     'EditableGraph',
     'parse_graph',
+    'ArcDiagram',
+    'InteractiveArcDiagram',
+    'MutableArcDiagram',
+    'EditableArcDiagram',
 ]
