@@ -18,7 +18,19 @@ from ._utils import (
 
 
 class PathPatchDataUnits(PathPatch):
-    # adapted from https://stackoverflow.com/a/42972469/2912349
+    """PathPatch in which the linewidth is also given in data units.
+
+    Parameters
+    ----------
+    *args, **kwargs
+        All arguments are passed through to PathPatch.
+
+    Notes
+    -----
+    Adapted from https://stackoverflow.com/a/42972469/2912349
+
+    """
+
     def __init__(self, *args, **kwargs):
         _lw_data = kwargs.pop("linewidth", 1)
         super().__init__(*args, **kwargs)
@@ -40,22 +52,23 @@ class PathPatchDataUnits(PathPatch):
 
 
 class NodeArtist(PathPatchDataUnits):
+    """Implements the node artists class.
+
+    Parameters
+    ----------
+    shape : str
+        The shape of the node. Specification is as for matplotlib.scatter marker, i.e. one of 'so^>v<dph8'.
+    xy : tuple
+        The (float x, float y) coordinates of the centroid.
+    radius : float
+        The distance from the center to each of the vertices.
+    **kwargs
+        `Patch` properties:
+        %(Patch_kwdoc)s
+
+    """
 
     def __init__(self, shape, xy, radius, **kwargs):
-        """
-        Parameters
-        ----------
-        shape : string
-            The shape of the node. Specification is as for matplotlib.scatter
-            marker, i.e. one of 'so^>v<dph8'.
-        xy : (float, float)
-            The center position.
-        radius : float
-            The distance from the center to each of the vertices.
-        **kwargs
-            `Patch` properties:
-            %(Patch_kwdoc)s
-        """
         self.shape = shape
         self.xy = xy
         self.radius = radius
@@ -115,7 +128,30 @@ class NodeArtist(PathPatchDataUnits):
 
 
 class EdgeArtist(PathPatchDataUnits):
+    """Implements the edge artist class.
 
+    Parameters
+    ----------
+    midline : ndarray
+        Array of (float x, float y) coordinates denoting the edge route.
+    width : float, default 0.05
+        The width of the edge (if shape is 'full').
+    head_width : float, default 0.10
+        Width of the arrow head.
+        Set to a value close to zero (but not zero) to suppress drawing of arrowheads.
+    head_length : float, default 0.15
+        Length of the arrow head.
+        Set to a value close to zero (but not zero) to suppress drawing of arrowheads.
+    offset : float, default 0.
+        For non-zero offset values, the end of the edge is offset from the target node.
+        The distance is calculated along the midline.
+    shape : {'full', 'left', 'right'}, default 'full'
+        The shape of the arrow.
+        For shapes 'left' and 'right' the arrow only one half of the arrow is plotted.
+    curved : bool, default False
+        Indicates if the midline is straight (False) or curved (True).
+
+    """
     def __init__(self, midline,
                  width       = 0.05,
                  head_width  = 0.10,
@@ -137,6 +173,7 @@ class EdgeArtist(PathPatchDataUnits):
         self._update_path() # sets self._path
         super().__init__(self._path, *args, **kwargs)
 
+
     def _update_path(self):
         # Determine the actual endpoint (and hence midline) of the arrow given the offset;
         # assume an ordered midline from source to target, i.e. from arrow base to arrow head.
@@ -217,6 +254,21 @@ class EdgeArtist(PathPatchDataUnits):
 
         self._path = Path(vertices, codes)
 
+
     def update_midline(self, midline):
+        """Update the midline and recompute the edge path."""
         self.midline = midline
         self._update_path()
+
+
+    def update_width(self, width, arrow=True):
+        """
+        Adjust the edge width. If arrow is True, the arrow head length and
+        width are rescaled by the ratio new width / old width.
+        """
+        if arrow:
+            ratio = width / self.width
+            self.head_length *= ratio
+            self.head_width *= ratio
+        self.width = width
+        self._update_path()
