@@ -10,7 +10,7 @@ import itertools
 import numpy as np
 
 from functools import wraps
-from itertools import combinations
+from itertools import combinations, product
 from scipy.spatial import Voronoi
 from rpack import pack
 
@@ -57,7 +57,12 @@ def _handle_multiple_components(layout_function):
                     components.append([node])
 
         if len(components) > 1:
-            return get_layout_for_multiple_components(edges, components, layout_function, *args, **kwargs)
+            if layout_function.__name__ == 'get_linear_layout':
+                return get_layout_for_multiple_components(
+                    edges, components, layout_function, mode='side-by-side', *args, **kwargs)
+            else:
+                return get_layout_for_multiple_components(
+                    edges, components, layout_function, mode='packed', *args, **kwargs)
         else:
             return layout_function(edges, *args, **kwargs)
 
@@ -65,9 +70,7 @@ def _handle_multiple_components(layout_function):
 
 
 def get_layout_for_multiple_components(edges, components, layout_function,
-                                       origin = (0, 0),
-                                       scale  = (1, 1),
-                                       *args, **kwargs):
+                                       origin, scale, mode='packed', *args, **kwargs):
     """Determine suitable bounding box dimensions and placement for each
     component in the graph, and then compute the layout of each
     individual component given the constraint of the bounding box.
@@ -81,10 +84,19 @@ def get_layout_for_multiple_components(edges, components, layout_function,
     layout_function : function
         Handle to the function computing the relative positions of each node within a component.
         The args and kwargs are passed through to this function.
-    origin : tuple, default (0, 0)
+    origin : tuple
         The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
-    scale : tuple, default (1, 1)
+    scale : tuple
         The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+    mode : str, default 'packed'
+        The way in which individual components are arranged w.r.t. each other.
+        One of:
+
+        - 'packed'       : Use rectangle packing to tightly arrange components.
+        - 'side-by-side' : Components are arranged next to each other.
+
+    *args, **kwargs
+        Passed through to layout_function
 
     Returns
     -------
@@ -93,7 +105,16 @@ def get_layout_for_multiple_components(edges, components, layout_function,
 
     """
 
-    bboxes = _get_component_bboxes(components, origin, scale)
+    if mode == 'packed':
+        bboxes = _get_packed_component_bboxes(components, origin, scale)
+    elif mode == 'side-by-side':
+        bboxes = _get_side_by_side_component_bboxes(components, origin, scale)
+    else:
+        msg = "Variable 'mode' is one of:"
+        msg += "\n    - 'packed'"
+        msg += "\n    - 'side-by-side'"
+        msg += f"\nCurrent value: '{mode}'"
+        raise ValueError(msg)
 
     node_positions = dict()
     for ii, (component, bbox) in enumerate(zip(components, bboxes)):
@@ -108,16 +129,18 @@ def get_layout_for_multiple_components(edges, components, layout_function,
     return node_positions
 
 
-def _get_component_bboxes(components, origin, scale, power=0.8, pad_by=0.05):
+def _get_packed_component_bboxes(components, origin, scale, power=0.8, pad_by=0.05):
     """Partition the canvas given by origin and scale into bounding boxes, one for each component.
 
+    Use rectangle packing to tightly arrange bounding boxes.
+
     Parameters
     ----------
     components : list of sets
         The connected components of the graph.
-    origin : tuple, default (0, 0)
+    origin : tuple
         The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
-    scale : tuple, default (1, 1)
+    scale : tuple
         The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
     power : float, default 0.8
         The dimensions each bounding box are given by |V|^power by |V|^power,
@@ -188,7 +211,61 @@ def _rescale_bboxes_to_canvas(bboxes, origin, scale):
     dimensions = [(w * scale_x, h * scale_y) for (_, _, w, h) in bboxes]
     rescaled_bboxes = [(x, y, w, h) for (x, y), (w, h) in zip(lower_left_hand_corners, dimensions)]
 
-    return rescaled_bboxes
+    # shift by origin
+    x0, y0 = origin
+    shifted_bboxes = [(x + x0, y + y0, w, h) for (x, y, w, h) in rescaled_bboxes]
+    return shifted_bboxes
+
+
+def _get_side_by_side_component_bboxes(components, origin, scale, pad_by=0.05):
+    """Partition the canvas given by origin and scale into bounding boxes, one for each component.
+
+    Position bounding boxes next to each other.
+
+    Parameters
+    ----------
+    components : list of sets
+        The connected components of the graph.
+    origin : tuple
+        The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
+    scale : tuple
+        The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+
+    Returns
+    -------
+    bboxes : list of tuples
+        The (left, bottom, width height) bounding boxes for each component.
+
+    """
+
+    relative_dimensions = [(len(component), 1) for component in components]
+
+    # Add a padding between boxes, such that nodes cannot end up touching in the final layout.
+    # We choose a padding proportional to the dimensions of the largest box.
+    maximum_dimensions = np.max(relative_dimensions, axis=0)
+    pad_x, pad_y = pad_by * maximum_dimensions
+    padded_dimensions = [(width + pad_x, height + pad_y) for (width, height) in relative_dimensions]
+
+    x = 0
+    origins = []
+    for (dx, _) in padded_dimensions:
+        origins.append((x, 0))
+        x += dx
+
+    bboxes = [(x, y, width, height) for (x, y), (width, height) in zip(origins, relative_dimensions)]
+
+    # rescale boxes to canvas, effectively reversing the upscaling
+    bboxes = _rescale_bboxes_to_canvas(bboxes, origin, scale)
+
+    if DEBUG:
+        import matplotlib.pyplot as plt
+        from matplotlib.patches import Rectangle
+        fig, ax = plt.subplots(1,1)
+        for bbox in bboxes:
+            ax.add_artist(Rectangle(bbox[:2], bbox[2], bbox[3], color=np.random.rand(3)))
+        plt.show()
+
+    return bboxes
 
 
 @_handle_multiple_components
@@ -599,7 +676,7 @@ class vertex_view(object):
 
 
 @_handle_multiple_components
-def get_circular_layout(edges, origin=(0,0), scale=(1,1), reduce_edge_crossings=True):
+def get_circular_layout(edges, origin=(0,0), scale=(1,1), node_order=None, reduce_edge_crossings=True):
     """Circular node layout.
 
     By default, this implementation uses a heuristic to arrange the nodes such that the edge crossings are minimised.
@@ -612,6 +689,9 @@ def get_circular_layout(edges, origin=(0,0), scale=(1,1), reduce_edge_crossings=
         The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
     scale : tuple
         The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+    node_order : list or None, default None
+        The (initial) ordering of nodes (counter-clockwise) before layout optimisation.
+        Set :code:`reduce_edge_crossings` to :code:`False` to skip optimisation and retain the given node order in the returned layout.
     reduce_edge_crossings : bool, default True
         If True, attempts to minimize edge crossings via the algorithm outlined in [Baur2005]_.
 
@@ -631,8 +711,13 @@ def get_circular_layout(edges, origin=(0,0), scale=(1,1), reduce_edge_crossings=
     radius *= 0.9 # fudge factor to make space for self-loops, annotations, etc
     positions = _get_n_points_on_a_circle(center, radius, len(nodes), start_angle=0)
 
+    if node_order:
+        nodes = [node for node in node_order if node in nodes]
+
     if reduce_edge_crossings:
         if not _is_complete(edges):
+            # remove self-loops as these should have no bearing on the solution
+            edges = [(source, target) for source, target in edges if source != target]
             nodes = _reduce_crossings(edges)
 
     return dict(zip(nodes, positions))
@@ -824,6 +909,210 @@ def _get_centroid(polygon):
     return np.mean(polygon, axis=0)
 
 
+@_handle_multiple_components
+def get_linear_layout(edges, origin=(0,0), scale=(1,1), node_order=None, reduce_edge_crossings=True):
+    """Linear node layout.
+
+    If :code:`reduce_edge_crossings` is set to :code:`True`, the algorithm
+    attempts to minimize edge crossings via the algorithm outlined in [Baur2005]_.
+
+    Parameters
+    ----------
+    edges : list
+        The edges of the graph, with each edge being represented by a (source node ID, target node ID) tuple.
+    origin : tuple, default (0, 0)
+        The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
+    scale : tuple, default (1, 1)
+        The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+    node_order : list or None, default None
+        The (initial) ordering of nodes (counter-clockwise) before layout optimisation.
+        Set :code:`reduce_edge_crossings` to :code:`False` to skip optimisation and retain the given node order in the returned layout.
+    reduce_edge_crossings : bool, default True
+        If True, attempts to minimize edge crossings via the algorithm outlined in [Baur2005]_.
+
+    Returns
+    -------
+    node_positions : dict
+        Dictionary mapping each node ID to (float x, float y) tuple, the node position.
+
+    References
+    ----------
+    .. [Baur2005] Baur & Brandes (2005) Crossing reduction in circular layouts.
+
+    """
+
+    nodes = _get_unique_nodes(edges)
+    total_nodes = len(nodes)
+    x = np.linspace(origin[0], origin[0] + scale[0], total_nodes+2)[1:-1]
+    y = np.full(total_nodes, origin[1] + 0.5 * scale[1])
+    positions = np.c_[x, y]
+
+    if node_order:
+        nodes = [node for node in node_order if node in nodes]
+
+    if reduce_edge_crossings:
+        if not _is_complete(edges):
+            # remove self-loops as these should have no bearing on the solution
+            edges = [(source, target) for source, target in edges if source != target]
+            nodes = _reduce_crossings(edges)
+
+            # The algorithm in _reduce_crossings assumes that nodes are arranged in a circle.
+            # As a consequence, when nodes are arranged along a line, a community may be split
+            # with members on both ends of the line.
+            # Here we find a better split by minimising the total edge length.
+            nodes = _minimize_total_edge_length(nodes, edges)
+
+    return dict(zip(nodes, positions))
+
+
+def _minimize_total_edge_length(nodes, edges):
+    total_iterations = len(nodes)
+    output = nodes
+    optimum = np.inf
+    for ii in range(total_iterations):
+        node_to_position = dict(zip(nodes, range(len(nodes))))
+        total_edge_length = np.sum([np.abs(node_to_position[source] - node_to_position[target]) for (source, target) in edges])
+        if total_edge_length < optimum:
+            output = nodes
+            optimum = total_edge_length
+        nodes = np.r_[nodes[1:], nodes[0]]
+    return output
+
+
+def get_bipartite_layout(edges, nodes=None, subsets=None, origin=(0, 0), scale=(1, 1), reduce_edge_crossings=True):
+    """Bipartite node layout.
+
+    By default, this implementation uses a heuristic to arrange the nodes such that the edge crossings are reduced.
+
+    Parameters
+    ----------
+    edges : list
+        The edges of the graph, with each edge being represented by a (source node ID, target node ID) tuple.
+    subsets : list
+        The two layers of the graph. If None, a two-coloring is used to separate the nodes into two subsets.
+        However, if the graph consists of multiple components, this partitioning into two layers is ambiguous, as multiple solutions exist.
+    origin : tuple
+        The (float x, float y) coordinates corresponding to the lower left hand corner of the bounding box specifying the extent of the canvas.
+    scale : tuple
+        The (float x, float y) dimensions representing the width and height of the bounding box specifying the extent of the canvas.
+    reduce_edge_crossings : bool, default True
+        If True, attempts to reduce edge crossings via the algorithm outlined in [Eades1994]_.
+
+    Returns
+    -------
+    node_positions : dict
+        Dictionary mapping each node ID to (float x, float y) tuple, the node position.
+
+    References
+    ----------
+    .. [Eades1994] Eades & Wormald (1994) Edge crossings in drawings of bipartite graphs.
+
+    """
+
+    adjacency_list = _edge_list_to_adjacency_list(edges, directed=False)
+
+    if nodes:
+        for node in nodes:
+            adjacency_list.setdefault(node, set())
+
+    if subsets:
+        left, right = subsets
+    else:
+        if len(_get_connected_components(adjacency_list)) > 1:
+            import warnings
+            msg = "The graph consistst of multiple components, and hence the partitioning into two subsets/layers is ambiguous!"
+            msg += "\n"
+            msg += "Use the `subsets` argument to explicitly specify the desired partitioning."
+            warnings.warn(msg)
+        left, right = _get_bipartite_sets(adjacency_list)
+
+    if reduce_edge_crossings:
+        if not _is_complete_bipartite(edges, left, right):
+            left, right = _reduce_crossings_bipartite(adjacency_list, left, right)
+
+    if len(left) > len(right):
+        spacing = scale[1] / (len(left) - 1)
+    else:
+        try:
+            spacing = scale[1] / (len(right) - 1)
+        except ZeroDivisionError:
+            # The graph consists of a single edge.
+            spacing = 1.
+
+    node_positions = dict()
+    for subset, xx in ((left, origin[0]), (right, origin[0] + scale[0])):
+        y = spacing * np.arange(len(subset))
+        y -= np.mean(y)
+        y += scale[1] / 2
+        for node, yy in zip(subset, y):
+            node_positions[node] = (xx, yy)
+
+    return node_positions
+
+
+def _get_bipartite_sets(adjacency_list):
+    colour = _get_two_colouring(adjacency_list)
+    left = [node for node in colour if colour[node] == 0]
+    right = [node for node in colour if colour[node] == 1]
+    return left, right
+
+
+def _get_two_colouring(adjacency_list):
+    # Adapted from networkx.algorithms.bipartite.color.
+    colour = dict()
+    for node in adjacency_list:
+        if (node in colour):
+            continue
+        elif len(adjacency_list[node]) == 0:
+            colour[node] = 0
+        else:
+            queue = [node]
+            colour[node] = 1
+            while queue:
+                node = queue.pop()
+                for neighbour in adjacency_list[node]:
+                    if neighbour in colour:
+                        if colour[neighbour] == colour[node]:
+                            raise Exception("Graph is not bipartite.")
+                    else:
+                        colour[neighbour] = 1 - colour[node]
+                        queue.append(neighbour)
+    return colour
+
+
+def _is_complete_bipartite(edges, left, right):
+    """Check if the bipartite graph is fully connected."""
+    minimal_complete_graph = list(product(left, right))
+
+    if len(edges) < len(minimal_complete_graph):
+        return False
+
+    for edge in minimal_complete_graph:
+        if (edge not in edges) and (edge[::-1] not in edges):
+            return False
+
+    return True
+
+
+def _reduce_crossings_bipartite(adjacency_list, left, right):
+    """Reduce the number of crossings in a bipartite graph using the median heuristic proposed in Eades & Wormald (1994)."""
+
+    left_ranks = {node : ii for ii, node in enumerate(left)}
+    right_ranks = dict()
+    for node in right:
+        neighbours = adjacency_list[node]
+        if neighbours:
+            right_ranks[node] = np.median([left_ranks[neighbour] for neighbour in neighbours])
+        else:
+            right_ranks[node] = 0
+
+    # TODO: break ties.
+    # If one node has an even number of neighbours and the other an odd number, than the odd one should have the lower rank.
+    # For the other two cases (even/even, odd/odd), no tie-break procedure seems specified in the paper.
+
+    return left, sorted(right_ranks, key=right_ranks.get)
+
+
 @_handle_multiple_components
 def get_community_layout(edges, node_to_community, origin=(0,0), scale=(1,1)):
     """Community node layout for modular graphs.
